AreOrthogonal := function(G1, G2)

# returns true if G1 and G2 are perpendicular

local sglg1, sglg2, i, sg, n1, A, B, perp;

#ensure that G1 and G2 are finite abelian groups 
if not IsGroup(G1) or not IsFinite(G1) or not IsAbelian(G1) or not IsGroup(G2) or not IsFinite(G2) or not IsAbelian(G2)then
  Error("Usage: AreOrthogonal(<G1>, <G2>) - where <G1>, <G2> are finite abelian groups\n");
fi;

perp := true;

sglg1 := List(LatticeSubgroups(G1)!.conjugacyClassesSubgroups, Representative);
sglg2 := List(LatticeSubgroups(G2)!.conjugacyClassesSubgroups, Representative);

# n1 is the number of divisors of |G1|
n1 := Size(DivisorsInt(Size(G1)));

for i in [1..n1] do
  for A in Filtered(sglg1, function(sg) return Size(sg) > 1 and Size(sg) = DivisorsInt(Size(G1))[i]; end) do
    for B in Filtered(sglg2, function(sg) return Size(sg) = Size(A); end) do
      if not IsomorphismGroups(A, B) = fail then
        perp := false;
      fi;
    od;
  od;
od;

return perp;

end;
