ClosedSubgroups := function(G) 

local   sgl, # list of all subgroups of G
      N_out, # N_out(x) - function returning the minimal supergroups of x
      minsl, # list containing the minimal supergroups relation for the whole subgroup lattice 
	     # (see MinimalSupergroupsLattice in the Reference Manual for details
      socle, # the socle of the group G
     marked, # boolean variable used for keeping track of already "visited" subgroups (vertices)
      queue, # list used to simulate a queue data structure 
     result, # the list of closed subgroups to be returned as result of the function
        atm, # a list used to store (for each subgroup) the number of atoms included
x, sg, t, c, atom, minsg, g; # auxiliary variables used in for loops etc.


# it is assumed that "minsl := MinimalSupergroupsLattice(LatticeSubgroups(G));" is called before 
# the first use of this function
N_out := function(x)
  return minsl[x]{[1..Size(minsl[x])]}[1];
end;

# ensure that the group G is finite and abelian
if not IsGroup(G) or not IsFinite(G) or not IsAbelian(G) then 
  Error("Usage: ClosedSubgroups(<G>) - where <G> is a finite abelian group\n");
fi;

# treat the trivial case
if G = TrivialSubgroup(G) then  return [ G ]; fi;

# obtain a list of all subgroups
sgl := List(LatticeSubgroups(G)!.conjugacyClassesSubgroups, Representative);

# compute the minimal supergroups relation (before any use of N_out !!!)
minsl := MinimalSupergroupsLattice(LatticeSubgroups(G));

# the trivial subgroup
t := Position(sgl, TrivialSubgroup(G));

atm := List([1..Size(sgl)], c -> []);

# every atom contains itself
for atom in N_out(t) do atm[atom] := [ atom ]; od;

# initialize the vector of "visited" subgroups
marked := List([1..Size(sgl)], c -> false);

queue := [ t ];

while not IsEmpty(queue) do 
  x := Remove(queue, 1);
  for sg in N_out(x) do
    UniteSet(atm[sg], atm[x]);
    if not marked[sg] then 
      Add(queue, sg);
      marked[sg] := true;
    fi;
  od;
  atm[x] := Size(atm[x]);
od;

return sgl{Filtered([1..Size(sgl)], x -> N_out(x) = [] or Minimum(atm{N_out(x)}) > atm[x])};

end;