Closures := function(G, A)

local   sgl, # list of all subgroups of G
      N_out, # N_out(x) - function returning the minimal supergroups of x
      minsl, # list containing the minimal supergroups relation for the whole subgroup lattice 
	     # (see MinimalSupergroupsLattice in the Reference Manual for details
     marked, # boolean variable used for keeping track of already "visited" subgroups (vertices)
      queue, # list used to simulate a queue data structure 
     result, # the list of closed subgroups to be returned as result of the function
        atm, # a list used to store (for each subgroup) the number of atoms included
x, sg, t, c, atom, minsg, g, a; # auxiliary variables used in for loops etc.


# it is assumed that "minsl := MinimalSupergroupsLattice(LatticeSubgroups(G));" is called before 
# the first use of this function
N_out := function(x)
  return minsl[x]{[1..Size(minsl[x])]}[1];
end;

#ensure that G and A are finite abelian groups and A is a subgroup of G
if not IsGroup(G) or not IsFinite(G) or not IsAbelian(G) or not IsSubgroup(G, A) then
  Error("Usage: Closures(<G>, <A>) - where <G> is a finite abelian group and <A> is a subgroup of <G>\n");
fi;

# treat the trivial case
if G = TrivialSubgroup(G) then  return [ G ]; fi;

# obtain a list of all subgroups
sgl := List(LatticeSubgroups(G)!.conjugacyClassesSubgroups, Representative);

# compute the minimal supergroups relation (before any use of N_out !!!)
minsl := MinimalSupergroupsLattice(LatticeSubgroups(G));

# the trivial subgroup
t := Position(sgl, TrivialSubgroup(G));

atm := List([1..Size(sgl)], c -> []);

# every atom contains itself
for atom in N_out(t) do atm[atom] := [ atom ]; od;

# initialize the vector of "visited" subgroups
marked := List([1..Size(sgl)], c -> false);

queue := [ t ];

while not IsEmpty(queue) do 
  x := Remove(queue, 1);
  for sg in N_out(x) do
    UniteSet(atm[sg], atm[x]);
    if not marked[sg] then 
      Add(queue, sg);
      marked[sg] := true;
    fi;
  od;
  atm[x] := Size(atm[x]);
od;

marked := List([1..Size(sgl)], c -> false);

a := Position(sgl, A);
queue := [ a ];
marked[a] := true;

result := [];

# Determine the maximal essential extensions of A
while not IsEmpty(queue) do 
  x := Remove(queue, 1);
  for sg in Filtered(N_out(x), function(sg) return not marked[sg] and atm[x] = atm[sg]; end) do 
    Add(queue, sg);
    marked[sg] := true;
  od;
  if Filtered(N_out(x), function(sg) return atm[x] = atm[sg]; end) = [] then Add(result, x); fi;
od;

return sgl{result};


end;