Coclosures := function(G, A)

local   sgl, # list of all subgroups of G
      N_out, # N_out(x) - function returning the minimal supergroups of x
      maxsl, # list containing the maximal subgroups relation for the whole subgroup lattice 
	     # (see MaximalSubgroupsLattice in the Reference Manual for details
     marked, # boolean variable used for keeping track of already "visited" subgroups (vertices)
      queue, # list used to simulate a queue data structure 
     result, # the list of closed subgroups to be returned as result of the function
       datm, # a list used to store (for each subgroup) the number of dual atoms in which is included
x, sg, t, c, datom, g, a; # auxiliary variables used in for loops etc.


# it is assumed that "maxsl := MaximalSubgroupsLattice(LatticeSubgroups(G));" is called before 
# the first use of this function
N_out := function(x)
  return maxsl[x]{[1..Size(maxsl[x])]}[1];
end;

#ensure that G and A are finite abelian groups and A is a subgroup of G
if not IsGroup(G) or not IsFinite(G) or not IsAbelian(G) or not IsSubgroup(G, A) then
  Error("Usage: Coclosures(<G>, <A>) - where <G> is a finite abelian group and <A> is a subgroup of <G>\n");
fi;

# treat the trivial case
if G = TrivialSubgroup(G) then  return [ G ]; fi;

# obtain a list of all subgroups
sgl := List(LatticeSubgroups(G)!.conjugacyClassesSubgroups, Representative);

# compute the maximal subgroups relation (before any use of N_out !!!)
maxsl := MaximalSubgroupsLattice(LatticeSubgroups(G));

g := Position(sgl, G);

datm := List([1..Size(sgl)], c -> []);

# every dual atom is contained in itself
for datom in N_out(g) do datm[datom] := [ datom ]; od;

# initialize the vector of "visited" subgroups
marked := List([1..Size(sgl)], c -> false);

queue := [ g ];

while not IsEmpty(queue) do 
  x := Remove(queue, 1);
  for sg in N_out(x) do
    UniteSet(datm[sg], datm[x]);
    if not marked[sg] then 
      Add(queue, sg);
      marked[sg] := true;
    fi;
  od;
  datm[x] := Size(datm[x]);
od;

marked := List([1..Size(sgl)], c -> false);

a := Position(sgl, A);
queue := [ a ];
marked[a] := true;

result := [];

# Determine the minimal coessential subgroups of A
while not IsEmpty(queue) do 
  x := Remove(queue, 1);
  for sg in Filtered(N_out(x), function(sg) return not marked[sg] and datm[x] = datm[sg]; end) do 
    Add(queue, sg);
    marked[sg] := true;
  od;
  if Filtered(N_out(x), function(sg) return datm[x] = datm[sg]; end) = [] then Add(result, x); fi;
od;

return sgl{result};


end;