CoessentialSubgroups := function(G, A)
local N_out, sgl, maxsl, marked, queue, sgA, S, Ca, x, sg, a;

# it is assumed that "maxsl := MaximalSubgroupsLattice(LatticeSubgroups(G));" is called before 
# the first use of this function
N_out := function(x)
  return maxsl[x]{[1..Size(maxsl[x])]}[1];
end;

#ensure that G and A are finite abelian groups and A is a subgroup of G
if not IsGroup(G) or not IsFinite(G) or not IsAbelian(G) or not IsSubgroup(G, A) then
  Error("Usage: CoessentialSubgroups(<G>, <A>) - where <G> is a finite abelian group and <A> is a subgroup of <G>\n");
fi;

Ca := [ A ];

# obtain a list of all subgroups
sgl := List(LatticeSubgroups(G)!.conjugacyClassesSubgroups, Representative);

# compute the maximal subgroups relation (before any use of N_out !!!)
maxsl := MaximalSubgroupsLattice(LatticeSubgroups(G));

# initialize the vector of "visited" subgroups
marked := List([1..Size(sgl)], c -> false);

# we determine now all the subgroups of A
a := Position(sgl, A);
queue := [ a ];
marked[a] := true;

sgA := [];

# Breadth-First Search for the subgroups of A
while not IsEmpty(queue) do 
  x := Remove(queue, 1);
  Add(sgA, x);
  for sg in Filtered(N_out(x), function(sg) return not marked[sg]; end) do 
    Add(queue, sg);
    marked[sg] := true;
  od;
od;

S := Difference(Intersection(SuperfluousSubgroups(G), sgl{sgA}), SuperfluousSubgroups(A));

for sg in S do Ca := Union(Ca, FullSummands(A, sg)); od;

return Ca;

end;
