Complements := function(G,A)
local     Za, # the set of all subgroups of G intersecting A in the trivial subgroup ("NullIntersectors")
       N_out, # N_out(x) - function returning the minimal supergroups of x       
         sgl, # list of all subgroups of G
       minsl, # list containing the minimal supergroups relation for the whole subgroup lattice 
	      # (see MinimalSupergroupsLattice in the Reference Manual for details
      marked, # boolean variable used for keeping track of already "visited" subgroups (vertices)
       queue, # list used to simulate a queue data structure 
      result, # the list of maximal "NullIntersector" subgroups to be returned as result of the function
    e, x, sg; # auxiliary variables used in for loops etc.

# it is assumed that "minsl := MinimalSupergroupsLattice(LatticeSubgroups(G));" is called before 
# the first use of this function
N_out := function(x)
  return minsl[x]{[1..Size(minsl[x])]}[1];
end;

#ensure that G and A are finite abelian groups and A is a subgroup of G
if not IsGroup(G) or not IsFinite(G) or not IsAbelian(G) or not IsSubgroup(G, A) then
  Error("Usage: Complements(<G>, <A>) - where <G> is a finite abelian group and <A> is a subgroup of <G>\n");
fi;

# determine the set of all subgroups of G intersecting A in the trivial subgroup
Za := NullIntersectors(G, A);

# make Za a sorted list (a set)
Sort(Za);

# obtain a list of all subgroups
sgl := List(LatticeSubgroups(G)!.conjugacyClassesSubgroups, Representative);

# compute the minimal supergroups relation (before any use of N_out !!!)
minsl := MinimalSupergroupsLattice(LatticeSubgroups(G));

# initialize the vector of "visited" subgroups
marked := List([1..Size(sgl)], c -> false);

e := Position(sgl, TrivialSubgroup(G));
queue := [ e ];
marked[e] := true;

result := [];

# Determine the maximal elements of Za
while not IsEmpty(queue) do 
  x := Remove(queue, 1);
  for sg in Filtered(N_out(x), function(sg) return not marked[sg] and sgl[sg] in Za; end) do 
    Add(queue, sg);
    marked[sg] := true;
  od;
  if Filtered(N_out(x), function(sg) return sgl[sg] in Za; end) = [] then Add(result, x); fi;
od;

return sgl{result};

end;
