EssentialSubgroups := function(G)
local   sgl, # list of all subgroups of G
      N_out, # N_out(x) - function returning the minimal supergroups of x
      minsl, # list containing the minimal supergroups relation for the whole subgroup lattice 
	     # (see MinimalSupergroupsLattice in the Reference Manual for details
      socle, # the socle of the group G
     marked, # boolean variable used for keeping track of already "visited" subgroups (vertices)
      queue, # list used to simulate a queue data structure 
     result, # the list of essential subgroups to be returned as result of the function
x, sg, i, c; # auxiliary variables used in for loops etc.

# it is assumed that "minsl := MinimalSupergroupsLattice(LatticeSubgroups(G));" is called before 
# the first use of this function
N_out := function(x)
  return minsl[x]{[1..Size(minsl[x])]}[1];
end;

# ensure that the group G is finite and abelian
if not IsGroup(G) or not IsFinite(G) or not IsAbelian(G) then 
  Error("Usage: EssentialSubgroups(<G>) - where <G> is a finite abelian group\n");
fi;

# treat the G = TrivialSubgroup(G) extreme case
if G = TrivialSubgroup(G) then return [ G ]; fi;

# obtain a list of all subgroups
sgl := List(LatticeSubgroups(G)!.conjugacyClassesSubgroups, Representative);

# compute the minimal supergroups relation (before any use of N_out !!!)
minsl := MinimalSupergroupsLattice(LatticeSubgroups(G));

# initialize the vector of "visited" subgroups
marked := List([1..Size(sgl)], c -> false);

# the socle of G is the first essential subgroup
socle := Position(sgl, Socle(G));
queue := [ socle ];
marked[socle] := true;

result := [];

# Breadth-First Search for the extensions of Socle(G)
while not IsEmpty(queue) do 
  x := Remove(queue, 1);
  Add(result, x);
  for sg in Filtered(N_out(x), function(sg) return not marked[sg]; end) do 
    Add(queue, sg);
    marked[sg] := true;
  od;
od;

return sgl{result};

end;