SuperfluousSubgroups := function(G)
local   sgl, # list of all subgroups of G
      N_out, # N_out(x) - function returning the maximal subgroups of x
      maxsl, # list containing the maximal subgroups relation for the whole subgroup lattice 
	     # (see MaximalSubgroupsLattice in the Reference Manual for details
    radical, # the radical of the group G
     marked, # boolean variable used for keeping track of already "visited" subgroups (vertices)
      queue, # list used to simulate a queue data structure 
     result, # the list of superfluous subgroups to be returned as result of the function
x, sg, i, c; # auxiliary variables used in for loops etc.

# it is assumed that "maxsl := MaximalSubgroupsLattice(LatticeSubgroups(G));" is called before 
# the first use of this function
N_out := function(x)
  return maxsl[x]{[1..Size(maxsl[x])]}[1];
end;

# ensure that the group G is finite and abelian
if not IsGroup(G) or not IsFinite(G) or not IsAbelian(G) then 
  Error("Usage: SuperfluousSubgroups(<G>) - where <G> is a finite abelian group\n");
fi;

# treat the G = TrivialSubgroup(G) extreme case
if G = TrivialSubgroup(G) then return [ G ]; fi;

# obtain a list of all subgroups
sgl := List(LatticeSubgroups(G)!.conjugacyClassesSubgroups, Representative);

# compute the maximal subgroups relation (before any use of N_out !!!)
maxsl := MaximalSubgroupsLattice(LatticeSubgroups(G));

# initialize the vector of "visited" subgroups
marked := List([1..Size(sgl)], c -> false);

# the radical of G is the first superfluous subgroup
radical := Position(sgl, Intersection(MaximalSubgroups(G)));
queue := [ radical ];
marked[radical] := true;

result := [];

# Breadth-First Search for the subgroups of Radical(G)
while not IsEmpty(queue) do 
  x := Remove(queue, 1);
  Add(result, x);
  for sg in Filtered(N_out(x), function(sg) return not marked[sg]; end) do 
    Add(queue, sg);
    marked[sg] := true;
  od;
od;

return sgl{result};

end;