Supplements := function(G,A)
local     Fa, # the set of all subgroups of G intersecting A in the trivial subgroup ("NullIntersectors")
       N_out, # N_out(x) - function returning the minimal supergroups of x       
         sgl, # list of all subgroups of G
       maxsl, # list containing the maximal subgroups relation for the whole subgroup lattice 
       	      # (see MaximalSubgroupsLattice in the Reference Manual for details
      marked, # boolean variable used for keeping track of already "visited" subgroups (vertices)
       queue, # list used to simulate a queue data structure 
      result, # the list of maximal "NullIntersector" subgroups to be returned as result of the function
    g, x, sg; # auxiliary variables used in for loops etc.

# it is assumed that "maxsl := MaximalSubgroupsLattice(LatticeSubgroups(G));" is called before 
# the first use of this function
N_out := function(x)
  return maxsl[x]{[1..Size(maxsl[x])]}[1];
end;

#ensure that G and A are finite abelian groups and A is a subgroup of G
if not IsGroup(G) or not IsFinite(G) or not IsAbelian(G) or not IsSubgroup(G, A) then
  Error("Usage: Supplements(<G>, <A>) - where <G> is a finite abelian group and <A> is a subgroup of <G>\n");
fi;

# determine the set of all subgroups of G intersecting A in the trivial subgroup
Fa := FullSummands(G, A);

# make Fa a sorted list (a set)
Sort(Fa);

# obtain a list of all subgroups
sgl := List(LatticeSubgroups(G)!.conjugacyClassesSubgroups, Representative);

# compute the maximal subgroups relation (before any use of N_out !!!)
maxsl := MaximalSubgroupsLattice(LatticeSubgroups(G));

# initialize the vector of "visited" subgroups
marked := List([1..Size(sgl)], c -> false);

g := Position(sgl, G);
queue := [ g ];
marked[g] := true;

result := [];

# Determine the minimal elements of Fa
while not IsEmpty(queue) do 
  x := Remove(queue, 1);
  for sg in Filtered(N_out(x), function(sg) return not marked[sg] and sgl[sg] in Fa; end) do 
    Add(queue, sg);
    marked[sg] := true;
  od;
  if Filtered(N_out(x), function(sg) return sgl[sg] in Fa; end) = [] then Add(result, x); fi;
od;

return sgl{result};

end;
